//Written by Varian Software Services NL
//Subject: Calendar component for Delphi 2.01
//Version: 1.02
//Date: 18 May 1997
//Last update: 12 June 1997
//Release: Freeware

//We try to build components for other people who may benefit from
//our efforts. No money is asked. If you've paid for this...You've paid
//too much. Just your comments and suggestions about improving our
//components are needed.

//Let's all work together in order to build a VCL worth having......

//if you make any modifications to the source, please send us a copy.
//We will verify your changes and give you proper credit when included.

//You've got stuck building your own components? Email us a copy and we will
//see if we can solve your problem(s), free of charge ofcourse.

//Sources and ideas for new components are welcom......

//Please send any questions, suggestions or remarks to our following
//address: Varian@worldaccess.nl

//Latest updates

//version 1.01
//-Added a ReadOnly property

//-Added a new property AlwaysShowSelect for displaying a
// 12 month calendar control/form without having selected cells
// in each component.

//-Added an data-aware calendar component for linking to the BDE

//-Tabstop didn't work. Fixed.

//Version 1.02
//-added fully configurable daynames header Visble/Color/Font
//idea from James Eckhardt <james@ne.com.au>

//-added public function DayOfTheYear

//-added public function WeekOfTheYear
//idea from Robert v.d. Zwaag <rjz@noord.bart.nl>

unit DateCtrl;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, DB, DBTables;

type
  TCalendarControl = class;

  TDayNames = class(TPersistent)
  private
    FOwner: TCalendarControl;
    FVisible: Boolean;
    FColor: TColor;
    FFont: TFont;
    procedure SetVisible(Value: Boolean);
    procedure SetColor(Value: TColor);
    procedure SetFont(Value: TFont);
  public
    constructor Create;
    destructor Destroy; override;
    procedure SetParent(Owner: TCalendarControl);
    procedure Assign(Source: TPersistent); override;
  published
    property Visible: Boolean read FVisible write SetVisible;
    property Color: TColor read FColor write SetColor;
    property Font: TFont read FFont write SetFont;
  end;

  TCalendarControl = class(TCustomControl)
  private
    FXCells: Integer;
    FYCells: Integer;
    FCellCount: Integer;
    FCellWidth: Integer;
    FCellHeight: Integer;
    FCurrent: Integer;
    FMonthOffset: Integer;
    FDrawGrid: Boolean;
    FHasFocus: Boolean;
    FYear, FMonth, FDay: Word;
    FButton: TMouseButton;
    FButtonDown: Boolean;
    FOnChange: TNotifyEvent;
    FWide3D: Boolean;
    FAlwaysShowSelect: Boolean;
    FReadOnly: Boolean;
    FDayNames: TDayNames;
    procedure UpdateCellSize;
    procedure GetCellRect(Wich: Integer; var R: TRect);
    function GetCellFromPos(X, Y: Integer): Integer;
    function GetCellText(Wich: Integer): string;
    procedure DrawCell(Wich: Integer; Contents: String; AState: Boolean);
    procedure SetDrawGrid(Value: Boolean);
    procedure SetWide3D(Value: Boolean);
    procedure SetShowDayNames(Value: Boolean);
    procedure SetAlwaysShowSelect(Value: Boolean);
    procedure SetReadOnly(Value: Boolean);
    procedure PaintGrid;
    procedure PaintCells(SignalChange: Boolean);
    procedure SetSelect(Wich: Integer);
    function GetFirstCell: Integer;
    function GetLastCell: Integer;
    function IsDayName(I: Integer): Boolean;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
  protected
    function IsLeapYear(AYear: Integer): Boolean;
    function DaysPerMonth(AYear, AMonth: Integer): Integer;
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Change;dynamic;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function DaysThisMonth: Integer;
    function ChangeDate(AYear, AMonth, ADay: Word): Boolean;
    function GetAsDateTime: TDateTime;
    function GetAsString(Format: String): String;
    function DayofTheYear: Integer;
    function WeekOfTheYear: Integer;
    procedure NextYear;
    procedure PrevYear;
    procedure NextMonth;
    procedure PrevMonth;
    property Day: Word read FDay;
    property Month: Word read FMonth;
    property Year: Word read FYear;
  published
    property AlwaysShowSelect: Boolean read FAlwaysShowSelect write SetAlwaysShowSelect;
    property Ctl3D;
    property Color;
    property DragCursor;
    property DragMode;
    property DrawGrid: Boolean read FDrawGrid write SetDrawGrid;
    property Enabled;
    property Font;
    property ReadOnly: boolean read FReadOnly write FReadOnly;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopUpMenu;
    property DayNames: TDayNames read FDayNames write FDayNames;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property Wide3D: Boolean read FWide3D write SetWide3D;
    property OnClick;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  TDBCalendarControl = class(TCalendarControl)
    FDataLink: TFieldDataLink;
  private
    procedure DataChange(Sender: TObject);
    procedure UpdateData(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Change; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property DataField: string  read GetDataField write SetDataField;
    property DataSource: TDataSource  read GetDataSource write SetDataSource;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Varian', [TCalendarControl, TDBCalendarControl]);
end;

function GetDayOfWeek(Year, Month, Day: Integer): Integer;
var
  Century, yr, dw: Integer;
begin
  if Month < 3 then
  begin
    Inc(Month, 10);
    Dec(Year);
  end else Dec(Month, 2);
  Century := Year div 100;
  yr := year mod 100;
  dw := (((26 * month - 2) div 10) + day + yr + (yr div 4) +
        (Century div 4) - (2 * Century)) mod 7;
  if dw < 0 then Result := dw + 7
  else Result := dw;
end;

function JulianDay(AYear, AMonth, ADay: Integer): Integer;
var
  Tmp1, Tmp2, Tmp3, Tmp4: Integer;
begin
  if AMonth > 2 then
  begin
    Tmp1 := AMonth - 3;
    Tmp2 := AYear;
  end
 else
  begin
    Tmp1 := AMonth + 9;
    Tmp2 := AYear - 1;
  end;
  Tmp1 := (Tmp1 * 153) + 2;
  Tmp3 := (Tmp2 div 100) * 146097;
  Tmp4 := (Tmp2 mod 100) * 1461;
  Result := (Tmp1 div 5) + (Tmp3 div 4) + (Tmp4 div 4) + ADay + 1721119;
end;

procedure JulianDayToDate(JDay: Integer; var AYear, AMonth, ADay: Integer);
var
  Tmp1, Tmp2, Tmp3: Integer;
begin
  Tmp1 := JDay - 1721119;
  Tmp3 := (4 * Tmp1 - 1) div 146097;
  Tmp1 := (4 * Tmp1 - 1) mod 146097;
  Tmp2 := Tmp1 div 4;
  Tmp1 := (4 * Tmp2 + 3) div 1461;
  Tmp2 := (4 * Tmp2 + 3) mod 1461;
  Tmp2 := (Tmp2 + 4) div 4;
  Amonth := (5 * Tmp2 - 3) div 153;
  Tmp2 := (5 * Tmp2 - 3) mod 153;
  ADay := ((Tmp2 + 5) div 5);
  AYear := (100 * Tmp3 + Tmp1);
  if AMonth < 10 then AMonth := AMonth + 3
 else
  begin
    AMonth := AMonth - 9;
    AYear := AYear + 1;
  end;
end;

function IsValidDate(AYear, AMonth, ADay: Integer): Boolean;
var
  JulDay: Integer;
  ycal, mcal, dcal : Integer;
begin
  JulDay := JulianDay(AYear, AMonth, ADay);
  JulianDayToDate(JulDay, ycal, mcal, dcal);
  Result := (AYear = ycal) and (AMonth = mcal) and (ADay = dcal)
end;

function WeekOfYear(Y, M, D: Integer): Integer;
var
  FirstJulian, JulDay: Integer;
  FirstDay: Integer;
begin
  JulDay := JulianDay(Y, M, D);
  FirstJulian := JulianDay(Y, 1, 1);
  FirstDay := GetDayOfWeek(Y, 1, 1);
  if FirstDay <> 0 then Inc(FirstJulian, 7-Ord(FirstDay));
  if JulDay < FirstJulian then Result := 0
 else Result := (JulDay - FirstJulian + 7) div 7;
end;


{TDayNames}
constructor TDayNames.Create;
begin
  Inherited Create;
  FVisible := true;
  FColor := clBtnFace;
  FFont := TFont.Create;
end;

destructor TDayNames.Destroy;
begin
  FFont.Free;
  Inherited Destroy;
end;

procedure TDayNames.SetParent(Owner: TCalendarControl);
begin
  FOwner := Owner;
end;

procedure TDayNames.SetVisible(Value: Boolean);
begin
  if FVisible <> Value then
  begin
    FVisible := Value;
    FOwner.SetShowDayNames(FVisible);
  end;
end;

procedure TDayNames.SetColor(Value: TColor);
begin
  if FColor <> Value then
  begin
    FColor := Value;
    FOwner.Invalidate;
  end;
end;

procedure TDayNames.SetFont(Value: TFont);
begin
  FFont.Assign(Value);
  FOwner.Invalidate;
end;

procedure TDayNames.Assign(Source: TPersistent);
begin
  FVisible := (Source as TDayNames).Visible;
  FColor := (Source as TDayNames).Color;
  FFont.assign( (Source as TDayNames).Font );
end;

{TCalendarControl}
constructor TCalendarControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque];
  SetBounds(0, 0, 250, 125);
  Canvas.Brush.Style := bsSolid;
  Canvas.Pen.Color := clBlack;
  Color := clBtnFace;
  Ctl3D := true;
  FXCells := 7;
  FYCells := 7;
  FDrawGrid := true;
  FWide3D := false;
  FCurrent := 0;
  FAlwaysShowSelect := true;
  FReadOnly := false;
  FDayNames := TDayNames.Create;
  FDayNames.SetParent(self);
  DecodeDate(Now, FYear, FMonth, FDay);
end;

destructor TCalendarControl.Destroy;
begin
  FDayNames.Free;
  Inherited Destroy;
end;

procedure TCalendarControl.UpdateCellSize;
var
  NewWidth, NewHeight: Integer;
begin
  FCellCount := FXCells * FYCells;
  NewWidth := (Width div FXCells) * FXCells;
  NewHeight := (Height div FYCells) * FYCells;
  BoundsRect := Bounds(Left, Top, NewWidth, NewHeight);
  FCellWidth := Width div FXCells;
  FCellHeight := Height div FYCells;
end;

procedure TCalendarControl.PaintGrid;
var
  I: Integer;
  R: TRect;
begin
  if not FDrawGrid then
    Exit;
  if Ctl3D then
  begin
    Canvas.Pen.Color := clBtnFace;
    Canvas.Brush.Color := Color;
    for I := 0 to FCellCount - 1 do
    begin
      GetCellRect(I, R);
      Canvas.Pen.Color := clBtnFace;
      Canvas.Brush.Color := Color;
      With R do Canvas.Rectangle(Left, Top, Right, Bottom);
      if FWide3D then InflateRect(R, -1, -1);
      Frame3D(Canvas, R, clBtnShadow, clBtnHighLight, 1);
    end;
  end
else
  begin
    with Canvas do
    begin
      Pen.Color := clBtnShadow;
      Pen.Width := 1;
      Brush.Color := Color;
      Rectangle(0, 0, Width, Height);
      for I := 0 to FXCells - 1 do
      begin
        MoveTo(I * FCellWidth, 0);
        LineTo(I * FCellWidth, Height);
      end;
      for I := 0 to FYCells - 1 do
      begin
        MoveTo(0, I * FCellHeight);
        LineTo(Width, I * FCellHeight);
      end;
    end;
  end;
end;

procedure TCalendarControl.PaintCells(SignalChange: Boolean);
var
  I: Integer;
  Selected: Boolean;
begin
  FMonthOffset := 1 - GetDayOfWeek(FYear, FMonth, 1);
  if (FDayNames.Visible) then Dec(FMonthOffset, FXCells);
  FCurrent := -FMonthOffset + FDay;
  for I := 0 to FCellCount - 1 do
  begin
    Selected := (FCurrent = I);
    if (not FAlwaysShowSelect) and (not FHasFocus) then
      Selected := false;
    DrawCell(I, GetCellText(I), Selected);
  end;
  if SignalChange then Change;
end;

procedure TCalendarControl.SetDrawGrid(Value: Boolean);
begin
  if FDrawGrid <> Value then
  begin
    FDrawGrid := Value;
    RecreateWnd;
  end;
end;

procedure TCalendarControl.SetWide3D(Value: Boolean);
begin
  if FWide3D <> Value then
  begin
    FWide3D := Value;
    RecreateWnd;
  end;
end;

procedure TCalendarControl.SetShowDayNames(Value: Boolean);
begin
  if (FDayNames.Visible) then FYCells := 7
                         else FYCells := 6;
  UpdateCellSize;
  RecreateWnd;
end;

procedure TCalendarControl.SetAlwaysShowSelect(Value: Boolean);
begin
  if (FAlwaysShowSelect <> Value) then
  begin
    FAlwaysShowSelect := Value;
    DrawCell(FCurrent, GetCellText(FCurrent), FAlwaysShowSelect);
  end;
end;

procedure TCalendarControl.SetReadOnly(Value: Boolean);
begin
  if (FReadOnly <> Value) then
  begin
    FReadOnly := Value;
    if (FAlwaysShowSelect = false) and (FReadOnly) then
      SetAlwaysShowSelect(true);
  end;
end;

procedure TCalendarControl.CMCtl3DChanged(var Message: TMessage);
begin
  inherited;
  RecreateWnd;
end;

procedure TCalendarControl.WMSize(var Message: TWMSize);
begin
  Inherited;
  UpdateCellSize;
end;

procedure TCalendarControl.GetCellRect(Wich: Integer; var R: TRect);
var
  X, Y: Integer;
begin
  X := (Wich mod FXCells) * FCellWidth;
  Y := (Wich div FXCells) * FCellHeight;
  R := Bounds(X, Y, FCellWidth, FCellHeight);
end;

function TCalendarControl.GetCellFromPos(X, Y: Integer): Integer;
var
  W, H: Integer;
begin
  W := (FCellWidth * FXCells) - 1;
  H := (FCellHeight * FYCells) - 1;
  if X > W then X := W else if X < 0 then X := 0;
  if Y > H then Y := H else if Y < 0 then Y := 0;
  {External error if result is loaded directly!!!  Compiler bug????}
  X := (Y div FCellHeight) * FXCells + (X div FCellWidth);
  Result := X;
end;

function TCalendarControl.GetCellText(Wich: Integer): string;
var
  DayNum: Integer;
begin
  DayNum := FMonthOffset + Wich;
  if (Wich < FXCells) and (FDayNames.Visible) then
    Result := ShortDayNames[Wich + 1]
  else
    if (DayNum < 1) or (DayNum > DaysThisMonth) then Result := ''
  else
    Result := IntToStr(DayNum);
end;

procedure TCalendarControl.DrawCell(Wich: Integer; Contents: String; AState: Boolean);
var
  R: TRect;
begin
  GetCellRect(Wich, R);
  InflateRect(R, -2, -2);

  Canvas.Font := Font;
  Canvas.Brush.Color := Color;
  if IsDayName(Wich) then
  begin
    Canvas.Font := FDayNames.Font;
    Canvas.Brush.Color := FDayNames.Color;
  end
 else
  if AState then
  begin
    Canvas.Brush.Color := clHighlight;
    Canvas.Font.Color := clHighlightText;
  end;

  with Canvas do
  begin
    TextRect(R, R.Left - 2 + (FCellWidth - TextWidth(Contents)) div 2,
      R.Top - 2 + (FCellHeight - TextHeight(Contents)) div 2, Contents);
    if (FHasFocus and AState) and (not FReadOnly) then
      DrawFocusRect(R);
  end;
end;

function TCalendarControl.IsDayName(I: Integer): Boolean;
begin
  Result := (I <= 6) and (FDayNames.Visible);
end;

function TCalendarControl.IsLeapYear(AYear: Integer): Boolean;
begin
  Result := (AYear mod 4 = 0) and ((AYear mod 100 <> 0) or (AYear mod 400 = 0));
end;

function TCalendarControl.DaysPerMonth(AYear, AMonth: Integer): Integer;
const
  DaysInMonth: array[1..12] of Integer = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
begin
  Result := DaysInMonth[AMonth];
  if (AMonth = 2) and IsLeapYear(AYear) then Inc(Result);
end;

function TCalendarControl.DaysThisMonth: Integer;
begin
  Result := DaysPerMonth(FYear, FMonth);
end;

procedure TCalendarControl.Paint;
var
  I: Integer;
begin
  PaintGrid;
  PaintCells(false);
end;

procedure TCalendarControl.WMSetFocus(var Message: TWMSetFocus);
begin
  FHasFocus := True;
  DrawCell(FCurrent, GetCellText(FCurrent), true);
  inherited;
end;

procedure TCalendarControl.WMKillFocus(var Message: TWMKillFocus);
begin
  FHasFocus := False;
  DrawCell(FCurrent, GetCellText(FCurrent), FAlwaysShowSelect);
  inherited;
end;

procedure TCalendarControl.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  Message.Result := DLGC_WANTARROWS or DLGC_WANTCHARS;
end;

procedure TCalendarControl.SetSelect(Wich: Integer);
begin
  if FReadOnly then
    Exit;
  if (FCurrent = Wich) or (GetCellText(Wich) = '') or
     ((Wich < FXCells) and (FDayNames.Visible)) then
    Exit;
  DrawCell(FCurrent, GetCellText(FCurrent), false);
  FCurrent := Wich;
  DrawCell(FCurrent, GetCellText(FCurrent), true);
  FDay := StrToInt(GetCellText(FCurrent));
  Change;
end;

procedure TCalendarControl.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  Cell: Integer;
begin
  inherited MouseDown(Button, Shift, X, Y);
  FButton := Button;
  FButtonDown := true;
  Cell := GetCellFromPos(X, Y);
  if Button = mbLeft then SetSelect(Cell);
  if TabStop then SetFocus;
end;

procedure TCalendarControl.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  Cell: Integer;
begin
  inherited MouseMove(Shift, X, Y);
  if FButtonDown then
  begin
    Cell := GetCellFromPos(X, Y);
    if FButton = mbLeft then SetSelect(Cell);
  end;
end;

procedure TCalendarControl.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  FButtonDown := False;
end;

procedure TCalendarControl.KeyDown(var Key: Word; Shift: TShiftState);
var
  NewSel: Integer;
begin
  inherited KeyDown(Key, Shift);
  NewSel := FCurrent;
  case Key of
    VK_HOME: NewSel := GetFirstCell;
    VK_END: NewSel := GetLastCell;
    VK_UP: if NewSel - FXCells >= GetFirstCell then Dec(NewSel, FXCells);
    VK_LEFT: if NewSel > GetFirstCell then Dec(NewSel);
    VK_DOWN: if (NewSel + FXCells <= GetLastCell) then Inc(NewSel, FXCells);
    VK_RIGHT: if NewSel < GetLastCell then Inc(NewSel);
  end;
  Key := 0;
  SetSelect(NewSel);
end;

function TCalendarControl.GetFirstCell: Integer;
begin
  Result := -FMonthOffset + 1;
end;

function TCalendarControl.GetLastCell: Integer;
begin
  Result := -FMonthOffset + DaysThisMonth;
end;

procedure TCalendarControl.Change;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

function TCalendarControl.ChangeDate(AYear, AMonth, ADay: Word): Boolean;
begin
  Result := IsValiddate(AYear, AMonth, ADay);
  if Result then
  begin
    FDay := ADay;
    FMonth := AMonth;
    FYear := AYear;
    PaintCells(true);
  end;
end;

function TCalendarControl.GetAsDateTime: TDateTime;
begin
  Result := EncodeDate(FYear, FMonth, FDay);
end;

function TCalendarControl.GetAsString(Format: String): String;
begin
  Result := FormatDateTime(Format, GetAsDateTime);
end;

procedure TCalendarControl.NextYear;
begin
  if IsLeapYear(FYear) and (FMonth = 2) and (FDay = 29) then FDay := 28;
  FYear := FYear + 1;
  PaintCells(true);
end;

procedure TCalendarControl.PrevYear;
begin
  if IsLeapYear(FYear) and (FMonth = 2) and (FDay = 29) then FDay := 28;
  FYear := FYear - 1;
  PaintCells(true)
end;

procedure TCalendarControl.NextMonth;
begin
  if (FMonth < 12) then Inc(FMonth)
 else
  begin
    FMonth := 1;
    FYear := FYear + 1;
  end;
  if FDay > DaysThisMonth then FDay := DaysThisMonth;
  PaintCells(true);
end;

procedure TCalendarControl.PrevMonth;
begin
  if (FMonth > 1) then Dec(FMonth)
 else
  begin
    FMonth := 12;
    FYear := FYear - 1;
  end;
  if FDay > DaysThisMonth then FDay := DaysThisMonth;
  PaintCells(true);
end;

function TCalendarControl.DayOfTheYear: Integer;
var
  yy, mm, dd, Tmp: Integer;
begin
  yy := FYear;
  mm := FMonth;
  dd := FDay;
  Tmp := (mm + 10) div 13;
  Result :=  3055 * (mm + 2) div 100 - Tmp * 2 - 91 +
             (1 - (yy - yy div 4 * 4 + 3) div 4 +
             (yy - yy div 100 * 100 + 99) div 100 -
             (yy - yy div 400 * 400 + 399) div 400) * Tmp + dd;
end;

function TCalendarControl.WeekOfTheYear: Integer;
begin
  Result := WeekOfYear(FYear, FMonth, FDay);
  if Result = 0 then
    Result := WeekOfYear(FYear - 1, 12, 31); {belongs to previous year}
end;


{TDBCalendarControl}
constructor TDBCalendarControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataLink := TFieldDataLink.Create;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnUpdateData := UpdateData;
end;

destructor TDBCalendarControl.Destroy;
begin
  FDataLink.Free;
  inherited Destroy;
end;

procedure TDBCalendarControl.DataChange(Sender: TObject);
var
  Y, M, D: Word;
begin
  if assigned(FDataLink.Field) then
  begin
    DecodeDate(FDataLink.Field.AsDateTime, Y, M, D);
    ChangeDate(Y, M, D);
  end;
end;

function TDBCalendarControl.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

function TDBCalendarControl.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TDBCalendarControl.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

procedure TDBCalendarControl.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

procedure TDBCalendarControl.UpdateData(Sender: TObject);
begin
  FDataLink.Field.AsDateTime := GetAsDateTime;
end;

procedure TDBCalendarControl.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  MyMouseDown: TMouseEvent;
begin
  if not ReadOnly and FDataLink.Edit then
    inherited MouseDown(Button, Shift, X, Y)
  else
  begin
    MyMouseDown := OnMouseDown;
    if Assigned(MyMouseDown) then MyMouseDown(Self, Button, Shift, X, Y);
  end;
end;

procedure TDBCalendarControl.KeyDown(var Key: Word; Shift: TShiftState);
var
  MyKeyDown: TKeyEvent;
begin
  if (not ReadOnly) and (FDataLink.Edit) then
    inherited KeyDown(Key, Shift)
  else
  begin
    MyKeyDown := OnKeyDown;
    if Assigned(MyKeyDown) then MyKeyDown(Self, Key, Shift);
  end;
end;

procedure TDBCalendarControl.Change;
begin
  FDataLink.Modified;
  inherited Change;
end;

procedure TDBCalendarControl.CMExit(var Message: TCMExit);
begin
  try
    FDataLink.UpdateRecord;
  except
    SetFocus;
    raise;
  end;
  inherited;
end;


end.
